#region usings
using System;
using System.ComponentModel.Composition;
using System.Runtime.InteropServices;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;

using VVVV.Core.Logging;
#endregion usings

namespace VVVV.Nodes
{
	#region PluginInfo
	[PluginInfo(Name = "K8055", Category = "IO", Version = "1.0", Help = "Basic template with one value in/out", Tags = "", AutoEvaluate = true)]
	#endregion PluginInfo
	public class C1_0IOK8055Node : IPluginEvaluate
	{
		#region dll
		        [DllImport("k8055d.dll")]
        public static extern int OpenDevice(int CardAddress);

        [DllImport("k8055d.dll")]
        public static extern void CloseDevice();

        [DllImport("k8055d.dll")]
        public static extern int ReadAnalogChannel(int Channel);

        [DllImport("k8055d.dll")]
        public static extern void ReadAllAnalog(ref int Data1, ref int Data2);

        [DllImport("k8055d.dll")]
        public static extern void OutputAnalogChannel(int Channel, int Data);

        [DllImport("k8055d.dll")]
        public static extern void OutputAllAnalog(int Data1, int Data2);

        [DllImport("k8055d.dll")]
        public static extern void ClearAnalogChannel(int Channel);

        [DllImport("k8055d.dll")]
        public static extern void SetAllAnalog();

        [DllImport("k8055d.dll")]
        public static extern void ClearAllAnalog();

        [DllImport("k8055d.dll")]
        public static extern void SetAnalogChannel(int Channel);

        [DllImport("k8055d.dll")]
        public static extern void WriteAllDigital(int Data);

        [DllImport("k8055d.dll")]
        public static extern void ClearDigitalChannel(int Channel);

        [DllImport("k8055d.dll")]
        public static extern void ClearAllDigital();

        [DllImport("k8055d.dll")]
        public static extern void SetDigitalChannel(int Channel);

        [DllImport("k8055d.dll")]
        public static extern void SetAllDigital();

        [DllImport("k8055d.dll")]
        public static extern bool ReadDigitalChannel(int Channel);

        [DllImport("k8055d.dll")]
        public static extern int ReadAllDigital();

        [DllImport("k8055d.dll")]
        public static extern int ReadCounter(int CounterNr);

        [DllImport("k8055d.dll")]
        public static extern void ResetCounter(int CounterNr);

        [DllImport("k8055d.dll")]
        public static extern void SetCounterDebounceTime(int CounterNr, int DebounceTime);

        [DllImport("k8055d.dll")]
        public static extern int Version();

        [DllImport("k8055d.dll")]
        public static extern int SearchDevices();

        [DllImport("k8055d.dll")]
        public static extern int SetCurrentDevice(int lngCardAddress);
		#endregion dll
		
		
		#region fields & pins
		
		// Inputs
		
		[Input("Card Address", DefaultValue = 0, IsSingle = true)]
		ISpread<int> s_addr;

		[Input("Enable", DefaultValue = 0, IsSingle = true)]
		ISpread<bool> enable;

		[Input("Digital Outputs", DefaultValue = 0)]
		ISpread<bool> dOut;

		[Input("Analog Outputs", DefaultValue = 0)]
		ISpread<double> aOut;

		[Input("Counter Reset", DefaultValue = 0)]
		ISpread<bool> ctrReset;

		
		// Outputs
		
		[Output("Digital Inputs")]
		ISpread<bool> dIn;

		[Output("Counters")]
		ISpread<int> ctr;

		[Output("Analog Inputs")]
		ISpread<double> aIn;

		[Import()]
		ILogger FLogger;
		#endregion fields & pins
		
		
		#region variables
		bool connected;
		#endregion variables
		
		
		//called each loop
		public void Evaluate(int SpreadMax)
		{
			// Connection
			if(enable[0]) connect();
			else disconnect();
			
			
			// OUTPUT
			
			// Write dOuts
			for(int i=0; i<8; i++){
				if(i < SpreadMax && dOut[i]) SetDigitalChannel(i+1);
				else ClearDigitalChannel(i+1);
			}
			
			// Write aOuts
			for(int i=0; i<2; i++){
				int i_aOut = (int) (aOut[i] * (float) 255);
				if(i >= SpreadMax) i_aOut = 0;
				if(i_aOut > 255) i_aOut = 255;
				if(i_aOut < 0) i_aOut = 0;
				OutputAnalogChannel(i+1, i_aOut);
			}

			// Reset Counters
			for(int i=0; i<2; i++){
				if(i >= SpreadMax) break;
				if(ctrReset[i]) ResetCounter(i + 1);
			}
			
			// Read dIns
			dIn.SliceCount = 5;
			int i_dIn = ReadAllDigital();
 			for(int i=0; i<5; i++){
				dIn[i] = (i_dIn & (1 << i)) > 0;
			}

			// Read Counters
			ctr.SliceCount = 2;
			for(int i=0; i<2; i++){
				ctr[i] = ReadCounter(i + 1);
			}
			
			// Read aIns
			aIn.SliceCount = 2;
			int i_aIn0 = 0, i_aIn1 = 0;
			ReadAllAnalog(ref i_aIn0, ref  i_aIn1);
 			aIn[0] = (double) i_aIn0 / (double) 255;
 			aIn[1] = (double) i_aIn1 / (double) 255;
		}
		
		public void connect(){
			if(connected) return;
			int addr;
			switch(s_addr[0]){
				case 0:
				case 1:
				case 2:
				case 3:
					addr = s_addr[0];
					break;
				default:
					addr = 0;
					break;
			}
            int h = OpenDevice(addr);
            switch (h) 
            {
            case  0:
            case  1:
            case  2:
            case  3: 
                FLogger.Log(LogType.Debug, "K8055 connected.");
            	connected = true;
            	break;
            case  -1 :
                FLogger.Log(LogType.Debug, "K8055 could not connect :(");
                break;
            }
		}
		public void disconnect(){
			if(!connected) return;
            FLogger.Log(LogType.Debug, "Closing connection to K8055.");
			CloseDevice();
			connected = false;
		}
	}
}
